from datetime import date
import pytest

from src.pyaemet.types_classes.observations import ObservationsDataFrame

from . import clima


@pytest.mark.parametrize("site, dates", [
    (["1111X"], [date(2020, 1, 1), date(2020, 2, 1)]),
    (["3100B"], [date(2020, 1, 1), date(2020, 2, 1)]),
    (["1111X"], [date(2020, 1, 1), date(2020, 2, 1)]),
    (["3100B"], [date(2020, 1, 1), date(2020, 2, 1)]),
    (["1111X"], [date(2020, 1, 1), date(2020, 2, 1)]),
    (["3100B", "1111X"], [date(2020, 1, 1), date(2020, 2, 1)]),
    ])
def test_daily_clima(site, dates):

    response = clima.daily_clima(site=site,
                                 start_dt=dates[0],
                                 end_dt=dates[1])

    assert isinstance(response, ObservationsDataFrame)
    assert not response.empty
    assert any(st in response.site.values for st in site)
    assert response.shape[0] == 32*len(site)



import pytest
from pandas import DataFrame

from .. import clima


@pytest.mark.parametrize("sites, codes", [
    ({"ciudad": ["Santander"]}, "1111X"),
    ({"ciudad": ["Santander"]}, "1111X"),
    ({"ciudad": ["Santander"]}, "1111X"),
    ({"ciudad": ["Santander"]}, "1111X"),
    ({"ciudad": ["Santander"]}, "1111X"),
    ])
def test_sites_in(sites, codes):

    response = clima.estaciones_loc(**sites, actualizar=False)

    assert isinstance(response, DataFrame)
    assert codes in response.indicativo.values
    assert response.shape == (3, 11)


def test_sites():

    response = clima.estaciones_info(actualizar=True)
    assert isinstance(response, DataFrame)
    assert not response.empty


def test_sites_near():
    response = clima.estaciones_cerca(latitud=43.47,
                                      longitud=-3.798,
                                      n_cercanas=4,
                                      actualizar=False,
                                      )

    print(type(response))
    assert isinstance(response, DataFrame)



from datetime import date
import pytest

from src.pyaemet.types_classes.sites import (
    SitesDataFrame,
    NearSitesDataFrame
    )

from . import clima


@pytest.mark.parametrize("sites, codes", [
    ({"city": ["Santander"]}, "1111X"),
    ({"city": ["Santander"]}, "1111X"),
    ({"city": ["Santander"]}, "1111X"),
    ({"city": ["Santander"]}, "1111X"),
    ({"city": ["Santander"]}, "1111X"),
    ])
def test_sites_in(sites, codes):

    response = clima.sites_in(**sites, update_first=False)

    assert isinstance(response, SitesDataFrame)
    assert codes in response.site.values
    assert response.shape == (3, 11)


def test_sites():

    response = clima.sites_info(update=True)

    assert isinstance(response, SitesDataFrame)


def test_sites_near():
    response = clima.near_sites(latitude=43.47,
                                longitude=-3.798,
                                n_near=4,
                                update_first = False,
                                )

    assert isinstance(response, NearSitesDataFrame)


def test_curation():
    sites = clima.near_sites(latitude=43.47,
                             longitude=-3.798,
                             n_near=4,
                             update_first = False,
                             )
    response = clima.sites_curation(
        start_dt=date.today(),
        end_dt=date.today(),
        sites=sites,
        threshold=0.75,
        variables='all',
        save_folder=None,
        )

    assert isinstance(response, SitesDataFrame)



"""
Data Curation
---------------

:author Jaimedgp
"""

import re
from datetime import time
from pandas import Series, isna, DataFrame


def update_fields(data_cols, metadata, new_metadata):
    """
    """

    for k, v in new_metadata.items():
        if k in data_cols:
            for i in metadata:
                if i["id"] == v["id"].lower():
                    v["aemet"] = {k: v for k, v in i.items() if k != "id"}
                    break

    return new_metadata


def _str_decima_notation(value):
    """ Docstring """

    if isna(value):
        return value

    # change decimal notation '0,0' => '0.0'
    return re.compile(r'(?<=\d),(?=\d)').sub(".", value)


def decimal_notation(column: Series):
    """ Docstring """

    if column.name not in ["date", "site"]:
        column = column.apply(_str_decima_notation)

    return column


def hr_to_datetime(value):
    """ Docstring """

    if isna(value):
        pass
    elif isinstance(value, str):
        if value == "-1":
            return time(0, 0, 59)
        elif value == "24":
            return time(hour=0,
                         minute=0,
                         second=0)
        elif ":" in value:
            hour, minute = map(int, value.split(":"))
            if (0 <= hour <= 23) and (0 <= minute <= 59):
                return time(hour=hour,
                            minute=minute,
                            second=0)
        else:
            if 0 <= int(value) <= 23:
                return time(hour=int(value),
                            minute=0,
                            second=0)

    return None


def convert_hours(column: Series):
    """ Docstring """

    if column.name.startswith("hr_"):
        column = column.apply(hr_to_datetime)

    return column


def remove_newline(data: DataFrame):

    return data.replace(r'\n', '', regex=True)
